import defu from "defu";
import { defineComponent, h } from "vue";
import { isVue2 } from "#imports";
const isSpan = (block) => block._type === "span";
const defaults = {
  types: {
    span: "span",
    image: "img"
  },
  marks: {
    strong: "strong",
    em: "em",
    link: "a"
  },
  styles: {
    h1: "h1",
    h2: "h2",
    h3: "h3",
    h4: "h4",
    h5: "h5",
    h6: "h6",
    normal: "p",
    blockquote: "blockquote"
  },
  listItem: "li",
  container: "div"
};
const validAttrs = [
  "abbr",
  "accesskey",
  "accessKey",
  "action",
  "alt",
  "autocomplete",
  "autofocus",
  "autoplay",
  "charset",
  "checked",
  "cite",
  "cols",
  "colspan",
  "command",
  "content",
  "datetime",
  "default",
  "disabled",
  "download",
  "draggable",
  "dropzone",
  "headers",
  "height",
  "hidden",
  "href",
  "hreflang",
  "id",
  "maxlength",
  "minlength",
  "muted",
  "placeholder",
  "preload",
  "radiogroup",
  "readonly",
  "required",
  "role",
  "selected",
  "src",
  "srcdoc",
  "srcset",
  "tabindex",
  "title",
  "value",
  "width",
  "wrap"
];
function findSerializer(item, serializers) {
  if (item?.listItem) {
    return serializers.listItem || "li";
  }
  return item?._type ? serializers.types[item._type] || serializers.marks[item._type] : void 0;
}
function renderStyle({ style, listItem }, serializers, children) {
  if (!listItem && style && serializers.styles[style]) {
    return h(serializers.styles[style], null, isVue2 ? children?.() : { default: children });
  }
  return children?.();
}
function renderInSerializer(item, serializers) {
  if (item._type !== "block")
    return render(serializers, item);
  return render(serializers, item, () => (item.children || []).map((child) => {
    if (isSpan(child)) {
      return renderMarks(child.text, child.marks, serializers, item.markDefs);
    }
    return render(serializers, child, () => renderMarks(child.text, child.marks, serializers, item.markDefs));
  }));
}
function renderMarks(content, [mark, ...marks] = [], serializers, markDefs = []) {
  if (!mark)
    return content;
  const definition = mark in serializers.marks ? { _type: mark, _key: "" } : markDefs.find((m) => m._key === mark);
  return render(serializers, definition, () => renderMarks(content, marks, serializers, markDefs));
}
function walkList(blocks, block) {
  if (!block.level) {
    blocks.push(block);
    return blocks;
  }
  const { _type, children, level } = blocks[blocks.length - 1] || {};
  if (_type === "list" && children) {
    if (level === block.level) {
      children.push(block);
    } else {
      walkList(children, block);
    }
  } else {
    blocks.push({
      _type: "list",
      children: [block],
      level: block.level
    });
  }
  return blocks;
}
function render(serializers, item, children) {
  const serializer = findSerializer(item, serializers);
  if (!serializer)
    return children?.();
  if (!item) {
    return void 0;
  }
  const isElement = typeof serializer === "string";
  const props = Object.fromEntries(Object.entries(item).filter(([key]) => key !== "_type").map(([key, value]) => {
    if (key === "_key")
      return ["key", value || null];
    if (!isElement || validAttrs.includes(key))
      return [key, value];
    return [];
  }));
  if (isElement) {
    return h(serializer, props, isVue2 ? children?.() : { default: () => children?.() });
  }
  return h(serializer, props, isVue2 ? children?.() : { default: () => children?.() });
}
function renderBlocks(blocks, serializers) {
  return blocks.map((block) => {
    const node = renderStyle(block, serializers, () => renderInSerializer(block, serializers));
    if (process.env.NODE_ENV === "development" && (!node || Array.isArray(node) && !node.length)) {
      console.warn(`No serializer found for block type "${block._type}".`, block);
    }
    return node;
  });
}
export default defineComponent({
  name: "SanityContent",
  props: {
    blocks: {
      type: Array,
      default: () => []
    },
    serializers: {
      type: Object,
      default: () => ({})
    }
  },
  setup(props) {
    const serializers = defu(props.serializers, defaults);
    serializers.types.list = serializers.types.list || createListSerializer(serializers);
    return () => renderBlocks(props.blocks?.reduce(walkList, []) || [], serializers);
  }
});
const createListSerializer = (serializers) => {
  return defineComponent({
    name: "ListComponent",
    inheritAttrs: false,
    props: {
      children: {
        type: Array,
        default: () => []
      }
    },
    setup(props) {
      return () => {
        const isOrdered = props.children[0]?.listItem === "number";
        return h(isOrdered ? "ol" : "ul", {}, isVue2 ? renderBlocks(props.children, serializers) : {
          default: () => renderBlocks(props.children, serializers)
        });
      };
    }
  });
};
